@@ -1,4 +1,4 @@
-# SPDX-FileCopyrightText: 2020 Jim Bennet for Adafruit Industries
+# SPDX-FileCopyrightText: 2020 Jim Bennett for Adafruit Industries
 # SPDX-FileCopyrightText: 2020 Elena Horton for Adafruit Industries
 #
 # SPDX-License-Identifier: MIT
@@ -136,7 +136,12 @@ class IoTCentralDevice(IoTMQTTCallback):
         :raises RuntimeError: if the internet connection is not responding or is unable to connect
         """
         self._device_registration = DeviceRegistration(
-            self._socket, self._id_scope, self._device_id, self._key, self._logger
+            self._socket,
+            self._iface,
+            self._id_scope,
+            self._device_id,
+            self._key,
+            self._logger,
         )
 
         token_expiry = int(time.time() + self._token_expires)
@@ -152,6 +157,10 @@ class IoTCentralDevice(IoTMQTTCallback):
             self._logger,
         )
 
+        self._logger.debug("Hostname: " + hostname)
+        self._logger.debug("Device Id: " + self._device_id)
+        self._logger.debug("Shared Access Key: " + self._key)
+
         self._mqtt.connect()
         self._mqtt.subscribe_to_twins()
 
